@@ -20,6 +20,7 @@ from os import path
 import os
 import time
 from calculate.core.server.gen_pid import search_worked_process
+from calculate.core.setup_cache import Cache as SetupCache
 from calculate.lib.cl_template import SystemIni
 from calculate.lib.datavars import DataVarsError
 
@@ -528,7 +529,6 @@ class Update(object):
         self.endTask()
         self.startTask(_("Fetching binary packages"))
 
-
     def _printUninstallPackage(self, pkg, num=1, max_num=1):
         """
         Вывод сообщения удаления пакета
@@ -728,6 +728,9 @@ class Update(object):
                     if emerge.install_packages.remove_list:
                         self.emerge_cache.drop_cache(
                             "List has packages for remove")
+                    elif emerge.install_packages.block_packages:
+                        self.emerge_cache.drop_cache(
+                            "List has block packages")
                     else:
                         self.updateCache(emerge.install_packages.list)
                 if not emerge.install_packages.list:
@@ -791,6 +794,9 @@ class Update(object):
         emerge.installing.add_observer(self._printInstallPackage)
         emerge.uninstalling.add_observer(self._printUninstallPackage)
         emerge.fetching.add_observer(self._printFetching)
+        def cancel_observing_fetch(fn):
+            emerge.fetching.clear_observers()
+        emerge.fetching.add_observer(cancel_observing_fetch)
         try:
             emerge.run()
         except EmergeError:
@@ -1044,3 +1050,8 @@ class Update(object):
                     except OSError:
                         pass
         return True
+
+    def updateSetupCache(self):
+        cache = SetupCache(self.clVars)
+        cache.update(force=True)
+        return True
